/****************************************************************************
 **
 ** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the examples of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:BSD$
 ** You may use this file under the terms of the BSD license as follows:
 **
 ** "Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that the following conditions are
 ** met:
 **   * Redistributions of source code must retain the above copyright
 **     notice, this list of conditions and the following disclaimer.
 **   * Redistributions in binary form must reproduce the above copyright
 **     notice, this list of conditions and the following disclaimer in
 **     the documentation and/or other materials provided with the
 **     distribution.
 **   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
 **     the names of its contributors may be used to endorse or promote
 **     products derived from this software without specific prior written
 **     permission.
 **
 ** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 ** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 ** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 ** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 ** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 ** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 ** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 ** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 ** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/

#include "HelpAssistant.h"

#include <QHelpContentWidget>
#include <QHelpEngine>
#include <QTextBrowser>

// This file is generated by the build system and will hold the version and some other things.
#include "BuildIfos.h"

namespace
{
class HelpBrowser : public QTextBrowser
{
public:
    HelpBrowser(QHelpEngine* helpEngine, QWidget* parent = nullptr)
        : QTextBrowser(parent),
          helpEngine(helpEngine)
    {
        helpEngine->setParent(this);
    }

    QVariant loadResource(int type, const QUrl& name) override
    {
        if (name.scheme() == "qthelp")
            return QVariant(helpEngine->fileData(name));
        else
            return QTextBrowser::loadResource(type, name);
    }

private:
    QHelpEngine* helpEngine;
};
} // namespace

HelpAssistant::HelpAssistant()
{
    createHelpWindow();
}

void HelpAssistant::showDocumentation(const QString &page)
{
    textViewer->setSource("qthelp://de.steckmann.logfileanalyser/doc/" + page);
    showMaximized();
}

void HelpAssistant::createHelpWindow()
{
    const auto appDir = QCoreApplication::applicationDirPath();
    QString helpFilename( "help.qhc" );
    QString helpFile(appDir + "/../" + globals::helpDirectory + "/" + helpFilename);

    if( !QFile::exists( helpFile ) )
    {
        helpFile = appDir + "/Help/" + helpFilename;
        if( !QFile::exists( helpFile ) )
        {
            qDebug() << "Unable to find helpfile: " << helpFilename;
        }
    }

    QHelpEngine* helpEngine = new QHelpEngine(helpFile);
    helpEngine->setupData();

    textViewer = new HelpBrowser(helpEngine);
    connect(helpEngine->contentWidget(),
            SIGNAL(linkActivated(QUrl)),
            textViewer,
            SLOT(setSource(QUrl)));
    setCentralWidget(textViewer);

    setMinimumSize({800, 600});
    setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
}
